% get_fixing_cov_constraints:
%
% Some covariances between a variable 'i' and 'n' - 1 other variables are 
% not free to be learned. If fixed(i, j) is true, then the covariance
% has to be equal to the respective value of covs for all non-redudant
% combinations of mixture indicators (mixture indicators of other variables
% different from i and j are set at level 1). These constraints have to
% be expressed in terms of Bartlett parameters. We assume that Bartlett
% parameters will be given a long row vector, where the first 'n'
% parameters corresponding to the regression coefficients of 'i' on the rest of
% 'subset' followed by the residual variance within the first group of
% instances that the mixture indicators can take. This is followed by the
% next 'n' Bartlett parameters according to the next mixture indicator
% instantiation and so and so forth (i.e., the parameter vector has 'n'
% times 'maximal_z' columns).
%
% The output is given by a matrix of constraints. In each row we have the
% coefficients of the constraint as required by MATLAB's optimization 
% routines. We also return the right-hand vector value for the constraints.
%
% Notice that 'i' here is an element of 'subset'.

function [Aeq beq] = get_fixing_cov_constraints(i, subset, fixed)

global covs; global vars; global pi_z; global graph;

% Initialization
n = length(subset);
i_pos = find(subset == i);
maximal_z = get_z_range(subset);

% Count number of constraints and do memory allocation
num_constraints = 0;
for j_pos = 1:n
    if j_pos == i_pos % variances are quadratic equalities, treated elsewhere
       continue
    end
    j = subset(j_pos);
    if ~fixed(i_pos, j_pos) && graph(i, j) ~= 0
        continue
    end
    % Notice this is also used to account for the structural zeros
    % Hence, use 'length(pi_z{i}) * length(pi_z{j})' instead of
    % 'length(covs{i, j})'.
    num_constraints = num_constraints + length(pi_z{i}) * length(pi_z{j}); 
end
if num_constraints > 0
   Aeq = zeros(num_constraints, n * maximal_z);
   beq = zeros(num_constraints, 1);
else
    Aeq = [];
    beq = [];
    return
end
num_constraints = 0;

% Now due to actual encoding of the constraints
for j_pos = 1:n
    if j_pos == i_pos % variances are quadratic equalities, treated elsewhere
       continue
    end
    j = subset(j_pos);
    if ~fixed(i_pos, j_pos) && graph(i, j) ~= 0
        continue
    end
    % All covariances between i and j have to be equal to the current value
    % set by gloval variable 'covs' across all instantiations of the
    % mixture indicators. Go then through all combinations of values for 
    % 'i' and 'j'. 
    for z_i = 1:length(pi_z{i})
        for z_j = 1:length(pi_z{j})
            
            % Get 'z_value', the reference class for the covariance of
            % 'i' and 'j' at mixture level 'z_i', 'z_j' by setting mixture
            % indicators of other variables to assume level 1 
            % (i.e., no need to worry about other cases where the mixture
            % indicators of other variables can take different values,
            % since we know the covariance of 'i', 'j' will be indifferent
            % to that.)
            
            z_values = ones(n, 1);
            z_values(i_pos) = z_i; z_values(j_pos) = z_j;
            z_value = get_z_inverse(subset, z_values); 
            p_index = get_parameter_index(i, j, z_values(i_pos), z_values(j_pos));

            % Now, encode this constraint: left-hand side
            num_constraints = num_constraints + 1;
            pos = (z_value - 1) * n + 1;
            for k_pos = 1:n
                if k_pos == i_pos
                    continue
                end
                k = subset(k_pos);
                if j ~= k
                   % The respective covariance of 'j' and 'k'.                   
                   if graph(j, k) ~= 0
                      pk_index = get_parameter_index(j, k, z_values(j_pos), z_values(k_pos));
                      Aeq(num_constraints, pos) = covs{j, k}(pk_index);
                   end
                else
                   % The respective variance of 'j'.
                   Aeq(num_constraints, pos) = vars{j}(z_j);
                end
                pos = pos + 1;
            end
            % Right-hand side:
            if graph(i, j) ~= 0
               beq(num_constraints) = covs{i, j}(p_index);            
            end
        end
    end        
end
