% initialize_from_latents:
%
% Fit a mixture model defined by the canonical latent variable model,
% then return the implied parameterization.
%
% Author: Ricardo Silva (ricardo@stats.ucl.ac.uk)
% Last modification: Oct 07 2008
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% For details in inputs and outputs, see 'learn.m'

function [means, covs, vars, pi, intercepts_out, coeffs_out, gammas_out, pi_z_out] = ...
         initialize_from_latents(graph, data, num_components, T, T_internal, tol)

global pi_z;

% Fit canonical latent variable model (one latent per clique). Number of
% different starting points is controlled by variable 'T_internal'

[intercepts_out, coeffs_out, gammas_out, pi_z_out] = ...
    lvm_learn(graph, data, num_components, T, T_internal, tol);
pi_z = pi_z_out;

% Convert latent variable parameterization into means and covariances

num_vars = length(graph);

means = cell(num_vars, 1);
covs = cell(num_vars, num_vars);
vars = cell(num_vars, 1);

[cliques, intersections, clique_parents] =  build_clique_information(graph);

coeff = cell(num_vars, 1);
for i = 1:num_vars
    coeff{i} = zeros(size(coeffs_out{i}));
    for v = 1:num_components(i)
        coeff{i}(:, v) = coeffs_out{i}(:, v);
    end
end

for i = 1:num_vars
    % create covariances
    for j = (i + 1):num_vars
        if graph(i, j) ~= 1
           continue;
        end
        covs{i, j} = zeros(length(pi_z_out{i}) * length(pi_z_out{j}), 1);
        total_v = length(covs{i, j});        
        for v = 1:total_v
            values = get_z([i j], v); % Safe, because i < j
            covs{i, j}(v) = 0.;
            for c = 1:length(intersections{i, j})
                cl = intersections{i, j}(c);
                cp1 = find(clique_parents{i} == cl, 1);
                if isempty(cp1)
                    error('clique information inconsistency error.');
                end
                cp2 = find(clique_parents{j} == cl, 1);
                if isempty(cp2)
                    error('clique information inconsistency error.');
                end
                factor = coeff{i}(cp1, values(1)) * coeff{j}(cp2, values(2));
                covs{i, j}(v) = covs{i, j}(v) + factor;
            end            
        end
        covs{j, i} = covs{i, j};
    end
    % create means and variances
    means{i} = zeros(num_components(i), 1);
    vars{i} = zeros(num_components(i), 1);    
    %y_c = kMeansCluster(data(:, i), num_components);
    for v = 1:num_components(i)
        vars{i}(v) = sum(coeff{i}(:, v).^2) + gammas_out{i}(v);
        %means{i}(v) = mean(y_c(y_c(:, 2) == v, 1));
        means{i}(v) = intercepts_out{i}(v);
    end
end

pi = pi_z_out;
