% rebuild_from_bartlett:
%
% From the Bartlett parameters b and gamma corresponding to the 
% conditional of Y_i given everybody else, rebuild the covariances of
% Y_i and others
%
% If type == 0, rebuild covariances and the variance of Y_i. If
% type == 1, only covariances. If type == 2, only variances
%
% Author: Ricardo Silva (ricardo@stats.ucl.ac.uk)
% Last modification: Oct 07 2008
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Inputs:
%
% - i: target variable

function rebuild_bartlett_partial(i, subset, b, gamma, code)

global vars; global covs; global graph;

num_spouses = length(subset) - 1;
spouses = setdiff(subset, i);

n = length(subset);

other_subset = setdiff(subset, i);
i_pos = find(subset == i);
other_pos = zeros(length(other_subset), 1);
for o = 1:length(other_subset)
    other_pos(o) = find(subset == other_subset(o));
end

% Rebuild covariances
if code == 1
for sp_j = 1:num_spouses
    j = spouses(sp_j);
    if graph(i, j) == 0
       continue;
    end
    j_pos = find(subset == j);
    num_v = length(covs{i, j});
    for v = 1:num_v        
        % First, find the proper choice of mixture indicator instantiation
        ij_values = get_z([i j], v);
        z_ref = ones(n, 1); 
        z_ref(subset == i) = ij_values(1); z_ref(subset == j) = ij_values(2);
        z_refp = get_z_inverse(subset, z_ref);
        
        % Second, find the covariances of 'j' and the other elements in
        % 'subset' within the proper mixture indicator instantiation
        other_covs = zeros(num_spouses, 1);
        for sp_s = 1:num_spouses
            s = spouses(sp_s);
            s_pos = find(subset == s);
            if s ~= j
               vs = get_parameter_index(s, j, z_ref(s_pos), z_ref(j_pos));
               if graph(s, j) ~= 0
                  other_covs(sp_s) = covs{s, j}(vs);
               end
            else
               other_covs(sp_s) = vars{s}(ij_values(2));
            end
        end
        
        % Finally, calculate the implied covariance
        imp_other_cov = assemble_cov(subset, z_ref); 
        implied  = b(:, z_refp)' * other_covs;
        
        %DEBUG 
        implied2 = b(:, z_refp)' * imp_other_cov(other_pos, j_pos);
        if abs(implied - implied2) > 1.e-6
            error('rebuild_from_bartlett: inconsistency Type I');
        end
        %END DEBUG
        
        v0 = get_parameter_index(i, j, z_ref(subset == i), z_ref(subset == j));
        covs{i, j}(v0) = implied;
        covs{j, i}(v0) = implied;
    end
end
end

% Rebuild variances

if code == 2
num_v = length(vars{i});
for v = 1:num_v
    z_ref = ones(n, 1); z_ref(subset == i) = v;
    z_refp = get_z_inverse(subset, z_ref);
        
    other_covs = zeros(num_spouses, 1);
    for sp_m = 1:num_spouses
        j = spouses(sp_m);
        if graph(i, j) ~= 0
           j_pos = find(subset == j);
           v_ij = get_parameter_index(i, j, z_ref(i_pos), z_ref(j_pos));        
           other_covs(sp_m) = covs{i, j}(v_ij);
        else
           other_covs(sp_m) = 0;
        end
    end
    implied = gamma(z_refp) + b(:, z_refp)' * other_covs;
    
    % DEBUG
    imp_other_cov = assemble_cov(other_subset, z_ref(other_pos)); 
    if det(imp_other_cov) < 0
       error('rebuild_from_bartlett: NeG here!');
    end
    implied2 = gamma(z_refp) + b(:, z_refp)' * imp_other_cov * b(:, z_refp);
    if abs(implied - implied2) > 1.e-6
       error('rebuild_from_bartlett: inconsistency Type II');
    end
    %END DEBUG
    
    vars{i}(v) = implied;
    
end
end
