% run:
%
% Learns a mixture of Gaussians using three different types of
% models: the factorial latent variable model, the factorial bi-directed 
% model and a standard mixture of Gaussians.
%
% Input parameters: 
%
% - train: data that will be used to fit the model.
% - test: optional. We will evaluate the predictive loglikelihood of the
%         models using this data.
% - graph: square matrix where '1' indicates edge and '0' indicates no
%          edge.
% - T:        number of iterations;
% - T_start:  number of random restarts;
% - K:        number of mixture components (per variable).
%
%
% Output:
%
% - means: the mixture component means for each variable;
% - vars: the mixture component variances for each variable;
% - covs: the mixture covariances for each variable. Since we are assuming
%         K levels per variable, there are K^2 entries for each covs{i, j}.
%         The order correspond to the following combination of mixture
%         levels for i < j: (1, 1), (2, 1), (3, 1)... (K, 1), (2, 1), (2,
%         2)... (K, 2), (3, 1)... (K, K).
% - pi_z: the marginal distribution of each mixture indicator.
% - intercepts: intercept parameters for the latent variable model.
% - coeffs: coefficient parameters for the latent variable model.
% - gammas: conditional variance parameters for the latent variable model.
% - pi_lvm: the marginal distribution of each mixture indicator for the 
%           latent variable model.
% - results: if a test set is given, this will return three values. The
%            first is the average predictive loglikelihood of the
%            bi-directed model, the second is the one corresponding to the
%            latent variable model, and the third a standard mixture of
%            Gaussians model.
%
% Send questions to Ricardo Silva (ricardo@stats.ucl.ac.uk)
% University College London, 2009.

function [means, vars, covs, pi_z, intercepts, coeffs, gammas, pi_lvm, results] = ...
         run(train, test, graph, T, T_start, K)

if size(train, 2) ~= size(graph, 2)
   error('Number of variables in data and graph should agree');
end
if ~isempty(test) && size(train, 2) ~= size(test, 2)
   error('Number of variables in training and test data should agree');
end

if size(train, 2) > 12
   disp('* * W A R N I N G * * This might run too slow or crash given the ' + ...
        'number of variables in your data. A maximum of 12 is recommended.');
end

if size(train, 2) > 25 % I'm being very optimistic here!
   disp('Too many variables! This will not run.');
end

tol = 1.e-4;
latent_init = true;
load_params = false;
use_relaxed = true;
num_components = K * ones(size(train, 2), 1);
params_name = 'learned';

[means, vars, covs, pi_z, intercepts, coeffs, gammas, pi_lvm] = ...
    learn(graph, train, num_components, latent_init, T, T_start, ...
          use_relaxed, params_name, load_params, tol);
llik = predictive_llik(test, means, vars, covs, pi_z, graph, K);
lvm_llik = lvm_predictive_llik(test, intercepts, coeffs, gammas, pi_lvm, graph, K) ;
mog_llik = learn_mog(train, test, K, T, 3 * T_start);
results = [llik; lvm_llik; mog_llik];
disp('Results: '); disp(results);
